@@ -281,30 +281,37 @@ foo(Foo) :- bar.
                    "foo(bar, (_->_;_), _):-_.")))
 
 (sweeprolog-deftest rename-variable ()
-  "Tests renaming varialbes."
+  "Tests renaming variables."
   "foo(Bar,Baz) :- spam(Baz,Bar)."
   (goto-char (point-min))
-  (sweeprolog-rename-variable "Bar" "Spam")
+  (sweeprolog-rename-variable "Bar" "_S_p_a_m_")
   (sweeprolog-rename-variable "Baz" "Bar")
-  (sweeprolog-rename-variable "Spam" "Baz")
+  (sweeprolog-rename-variable "_S_p_a_m_" "Baz")
   (should (string= (buffer-string)
                    "foo(Baz,Bar) :- spam(Bar,Baz).")))
 
 (sweeprolog-deftest increment-variable ()
-  "Tests renaming varialbes."
+  "Test incrementing index of numbered variables."
   "
 foo(Bar0,Bar) :-
+    spam(Bar_0,Bar_1),
     spam(Bar0,Bar1),
+    bar(Bar_1,Bar_2),
     bar(Bar1,Bar2),
+    baz(Bar_2,Bar),
     baz(Bar2, Bar).
 "
   (goto-char (1+ (point-min)))
   (sweeprolog-increment-numbered-variables 1 (point) "Bar1")
+  (sweeprolog-increment-numbered-variables 1 (point) "Bar_2")
   (should (string= (buffer-string)
                    "
 foo(Bar0,Bar) :-
+    spam(Bar_0,Bar_1),
     spam(Bar0,Bar2),
+    bar(Bar_1,Bar_3),
     bar(Bar2,Bar3),
+    baz(Bar_3,Bar),
     baz(Bar3, Bar).
 ")))
 
@@ -1846,6 +1853,21 @@ body.
 head,
   right_hand_context -->
     body.
+")
+  (sweeprolog-test-indentation
+   "/** <module> Foo
+
+This module provides foo bar
+ baz spam
+  foo bar baz
+*/
+"
+   "/** <module> Foo
+
+This module provides foo bar
+baz spam
+foo bar baz
+*/
 "))
 
 (sweeprolog-deftest forward-sexp-with-adjacent-operators ()
@@ -2426,5 +2448,42 @@ foo(Bar) --> baz(Bar).
     Skip zero or more white-space characters.
 " nil))))
 
+(sweeprolog-deftest eldoc-ssu-guard ()
+  "Test `sweep_short_documentation/2' with SSU guard."
+  "
+:- module(eldocssuguard, []).
+
+:- use_module(library(lists)).
+"
+  (should (equal (sweeprolog--query-once
+                  "sweep" "sweep_short_documentation"
+                  (list "foo(X), member(X,_) => true." 15 (buffer-file-name)))
+                 '("lists:member/2" "member(?Elem,?List) is unspec.
+    True if Elem is a member of List.
+"
+                   (7 . 12)))))
+
+
+(sweeprolog-deftest eldoc-dcg-pushback ()
+  "Test `sweep_short_documentation/2' with DCG pushback list."
+  "
+:- module(eldocdcgpushback, []).
+
+%!  foo(-Baz:string)// is det.
+%
+%   Doit.
+
+foo(_) --> [].
+
+"
+  (should (equal (sweeprolog--query-once
+                  "sweep" "sweep_short_documentation"
+                  (list "foo(X), [1,2,3] --> []." 5 (buffer-file-name)))
+                 '("eldocdcgpushback:foo//1" "foo(-Baz:string)// is det.
+    Doit.
+"
+                   (4 . 15)))))
+
+
 
 ;;; sweeprolog-tests.el ends here
