@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2013-2022, VU University Amsterdam
+    Copyright (c)  2013-2024, VU University Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
@@ -86,6 +86,28 @@ dict_functor(int pairs)
   }
 }
 
+static bool
+is_dict_functor(functor_t f)
+{ FunctorDef fd = valueFunctor(f);
+
+  return ( fd->name == ATOM_dict &&
+	   fd->arity%2 == 1 );
+}
+
+#define is_dict_term(w) \
+	LDFUNC(is_dict_term, w)
+
+static bool
+is_dict_term(DECL_LD word w)
+{ if ( isTerm(w) )
+  { Functor f = valueTerm(w);
+
+    return is_dict_functor(f->definition);
+  }
+
+  return false;
+}
+
 		 /*******************************
 		 *      LOW-LEVEL FUNCTIONS	*
 		 *******************************/
@@ -98,20 +120,18 @@ get_dict_ex(DECL_LD term_t t, Word dp, int ex)
   deRef(p);
   if ( isTerm(*p) )
   { Functor f = valueTerm(*p);
-    FunctorDef fd = valueFunctor(f->definition);
 
-    if ( fd->name == ATOM_dict &&
-	 fd->arity%2 == 1 )		/* does *not* validate ordering */
+    if ( is_dict_functor(f->definition) )
     { *dp = *p;
-      return TRUE;
+      return true;
     }
   }
 
   if ( !ex )
-    return FALSE;
+    return false;
 
   PL_type_error("dict", t);
-  return FALSE;
+  return false;
 }
 
 
@@ -129,18 +149,16 @@ get_create_dict_ex(DECL_LD term_t t, term_t dt)
   deRef(p);
   if ( isTerm(*p) )
   { Functor f = valueTerm(*p);
-    FunctorDef fd = valueFunctor(f->definition);
 
-    if ( fd->name == ATOM_dict &&
-	 fd->arity%2 == 1 )		/* does *not* validate ordering */
+    if ( is_dict_functor(f->definition) )
     { *valTermRef(dt) = *p;
-      return TRUE;
+      return true;
     }
   }
 
   if ( PL_get_dict_ex(t, 0, dt, DICT_GET_ALL) )
   { assert(isTerm(*valTermRef(dt)));
-    return TRUE;
+    return true;
   }
 
   return PL_type_error("dict", t);
@@ -185,8 +203,8 @@ dict_lookup_ptr(DECL_LD word dict, word name)
 
 /* True if the keys are proper keys and ordered.  Return values:
 
-   TRUE:  correctly ordered dict
-   FALSE: not ordered
+   true:  correctly ordered dict
+   false: not ordered
    -1:    not a key
    -2:    duplicate key
 */
@@ -194,7 +212,7 @@ dict_lookup_ptr(DECL_LD word dict, word name)
 #define dict_ordered(data, count, dupl) LDFUNC(dict_ordered, data, count, dupl)
 static int
 dict_ordered(DECL_LD Word data, int count, Word dupl)
-{ int ordered = TRUE;
+{ int ordered = true;
   Word n1, n2;
 
   if ( count > 0 )
@@ -210,7 +228,7 @@ dict_ordered(DECL_LD Word data, int count, Word dupl)
       if ( *n1 < *n2 )
 	continue;
       if ( *n1 > *n2 )
-	ordered = FALSE;
+	ordered = false;
       if ( *n1 == *n2 )
       { if ( dupl )
 	{ *dupl = *n1;
@@ -330,7 +348,7 @@ assign_in_dict(DECL_LD Word dp, Word val)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -340,13 +358,13 @@ put_dict(DECL_LD word dict, int size, Word nv, word *new_dict)
 { Functor data = valueTerm(dict);
   int arity = arityFunctor(data->definition);
   Word new, out, in, in_end, nv_end;
-  int modified = FALSE;
+  int modified = false;
 
   assert(arity%2 == 1);
 
   if ( size == 0 )
   { *new_dict = dict;
-    return TRUE;
+    return true;
   }
 
   if ( gTop+1+arity+2*size > gMax )
@@ -366,11 +384,11 @@ put_dict(DECL_LD word dict, int size, Word nv, word *new_dict)
     deRef2(nv+1, n_name);
 
     if ( *i_name == *n_name )
-    { if ( (rc=assign_in_dict(out++, nv)) != TRUE )
+    { if ( (rc=assign_in_dict(out++, nv)) != true )
 	return rc;
       *out++ = *i_name;
-      if ( !modified && compareStandard(nv, in, TRUE) )
-	modified = TRUE;
+      if ( !modified && compareStandard(nv, in, true) )
+	modified = true;
       in += 2;
       nv += 2;
     } else if ( *i_name < *n_name )
@@ -378,18 +396,18 @@ put_dict(DECL_LD word dict, int size, Word nv, word *new_dict)
       *out++ = *i_name;
       in += 2;
     } else
-    { if ( (rc=assign_in_dict(out++, nv)) != TRUE )
+    { if ( (rc=assign_in_dict(out++, nv)) != true )
 	return rc;
       *out++ = *n_name;
       nv += 2;
-      modified = TRUE;
+      modified = true;
     }
   }
 
   if ( nv == nv_end )
   { if ( !modified )
     { *new_dict = dict;
-      return TRUE;
+      return true;
     }
     while(in < in_end)
     { Word i_name;
@@ -404,7 +422,7 @@ put_dict(DECL_LD word dict, int size, Word nv, word *new_dict)
       int rc;
 
       deRef2(nv+1, n_name);
-      if ( (rc=assign_in_dict(out++, nv)) != TRUE )
+      if ( (rc=assign_in_dict(out++, nv)) != true )
 	return rc;
       *out++ = *n_name;
       nv += 2;
@@ -417,7 +435,70 @@ put_dict(DECL_LD word dict, int size, Word nv, word *new_dict)
 
   *new_dict = consPtr(new, TAG_COMPOUND|STG_GLOBAL);
 
-  return TRUE;
+  return true;
+}
+
+
+#define copy_keys_dict(dict, new_dict) \
+	LDFUNC(copy_keys_dict, dict, new_dict)
+
+static int
+copy_keys_dict(DECL_LD word dict, word *new_dict)
+{ Functor data = valueTerm(dict);
+  size_t arity = arityFunctor(data->definition);
+  Word new, out, in, in_end;
+
+  if ( gTop+1 > gMax )
+    return GLOBAL_OVERFLOW;
+
+  new    = gTop;
+  out    = new;
+  *out++ = data->definition;
+  setVar(*out++);
+  in     = data->arguments+1;
+  in_end = in+arity-1;
+
+  while(in < in_end)
+  { Word i_name;
+
+    deRef2(in+1, i_name);
+    setVar(*out++);
+    *out++ = *i_name;
+    in += 2;
+  }
+
+  gTop = out;
+  *new_dict = consPtr(new, TAG_COMPOUND|STG_GLOBAL);
+
+  return true;
+}
+
+#define same_keys_dict(dict1, dict2) \
+	LDFUNC(same_keys_dict, dict1, dict2)
+
+static bool
+same_keys_dict(DECL_LD word dict1, word dict2)
+{ Functor data1 = valueTerm(dict1);
+  Functor data2 = valueTerm(dict2);
+  size_t arity = arityFunctor(data1->definition);
+
+  if ( data1->definition != data2->definition )
+    return false;
+
+  Word d1 = data1->arguments+1;
+  Word d2 = data2->arguments+1;
+  Word dend = d1+arity-1;
+
+  for(; d1 < dend; d1+=2, d2+=2)
+  { Word k1, k2;
+
+    deRef2(d1+1, k1);
+    deRef2(d2+1, k2);
+    if ( *k1 != *k2 )
+      return false;
+  }
+
+  return true;
 }
 
 
@@ -455,12 +536,12 @@ del_dict(DECL_LD word dict, word key, word *new_dict)
 
   *new_dict = consPtr(new, TAG_COMPOUND|STG_GLOBAL);
 
-  return TRUE;
+  return true;
 }
 
 
 /* partial_unify_dict(dict1, dict2) unifies all common elements of two
-   dicts.  It returns TRUE on success, FALSE on a failed unification
+   dicts.  It returns true on success, false on a failed unification
    and *_OVERFLOW on some memory overflow
 */
 
@@ -476,7 +557,7 @@ partial_unify_dict(DECL_LD word dict1, word dict2)
   int rc;
 
   /* unify the tages */
-  if ( (rc=unify_ptrs(in1, in2, ALLOW_RETCODE)) != TRUE )
+  if ( (rc=unify_ptrs(in1, in2, ALLOW_RETCODE)) != true )
     return rc;
 
   /* advance to first v+k entry */
@@ -489,7 +570,7 @@ partial_unify_dict(DECL_LD word dict1, word dict2)
     deRef2(in1+1, n1);
     deRef2(in2+1, n2);
     if ( *n1 == *n2 )
-    { if ( (rc = unify_ptrs(in1, in2, ALLOW_RETCODE)) != TRUE )
+    { if ( (rc = unify_ptrs(in1, in2, ALLOW_RETCODE)) != true )
 	return rc;
       in1 += 2;
       in2 += 2;
@@ -500,7 +581,7 @@ partial_unify_dict(DECL_LD word dict1, word dict2)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -521,11 +602,11 @@ select_dict(DECL_LD word del, word from, word *new_dict)
   Word fin  = fd->arguments;
   Word dend = din+arityFunctor(dd->definition);
   Word fend = fin+arityFunctor(fd->definition);
-  size_t left = 0;
+  Word out0 = gTop, out = out0+2; /* dict functor and var */
   int rc;
 
   /* unify the tags */
-  if ( (rc=unify_ptrs(din, fin, ALLOW_RETCODE)) != TRUE )
+  if ( (rc=unify_ptrs(din, fin, ALLOW_RETCODE)) != true )
     return rc;
 
   /* advance to first v+k entry */
@@ -538,57 +619,47 @@ select_dict(DECL_LD word del, word from, word *new_dict)
     deRef2(din+1, d);
     deRef2(fin+1, f);
 
-    if ( *d == *f )
-    { if ( (rc = unify_ptrs(din, fin, ALLOW_RETCODE)) != TRUE )
+    if ( *d == *f )		/* same keys */
+    { if ( (rc = unify_ptrs(din, fin, ALLOW_RETCODE)) != true )
 	return rc;
       din += 2;
       fin += 2;
     } else if ( *d < *f )
-    { return FALSE;
+    { return false;
     } else
-    { fin += 2;
-      left++;
+    { if ( new_dict )
+      { if ( out+2 <= gMax )
+	{ *out++ = linkValI(fin);
+	  *out++ = *f;
+	} else
+	  return GLOBAL_OVERFLOW;
+      }
+      fin += 2;
     }
   }
   if ( din < dend )
-    return FALSE;
-  left += (fend-fin)/2;
+  { return false;
+  } else if ( new_dict )
+  { if ( out+(fend-fin) <= gMax )
+    { while(fin < fend)
+      { Word f;
 
-  if ( !new_dict )
-    return TRUE;
-
-  if ( gTop+2+2*left <= gMax )
-  { Word out = gTop;
-
-    *new_dict = consPtr(out, TAG_COMPOUND|STG_GLOBAL);
-
-    *out++ = dict_functor(left);
-    setVar(*out++);			/* tag for new dict */
-
-    din = dd->arguments+1;
-    fin = fd->arguments+1;
-
-    while(left > 0)
-    { Word d, f;
-
-      deRef2(din+1, d);
-      deRef2(fin+1, f);
-      if ( *d == *f )
-      { din += 2;
-	fin += 2;
-      } else
-      { *out++ = linkValI(fin);
+	deRef2(fin+1, f);
+	*out++ = linkValI(fin);
 	*out++ = *f;
 	fin += 2;
-	left--;
       }
-    }
-    gTop = out;
 
-    return TRUE;
-  }
-
-  return GLOBAL_OVERFLOW;
+      *new_dict = consPtr(out0, TAG_COMPOUND|STG_GLOBAL);
+      size_t nsz = (out-out0-2)/2;
+      *out0++ = dict_functor(nsz);
+      setVar(*out0);		/* out dict is anonymous */
+      gTop = out;
+      return true;
+    } else
+      return GLOBAL_OVERFLOW;
+  } else
+    return true;
 }
 
 
@@ -600,11 +671,11 @@ get_name_ex(DECL_LD term_t t, Word np)
   deRef(p);
   if ( is_dict_key(*p) )
   { *np = *p;
-    return TRUE;
+    return true;
   }
 
   PL_type_error("dict-key", t);
-  return FALSE;
+  return false;
 }
 
 
@@ -629,13 +700,13 @@ get_name_value(DECL_LD Word p, Word name, Word value, mark *m, int flags)
 	deRef2(&f->arguments[1], vp);
 	*value = linkValI(vp);
 
-	return TRUE;
+	return true;
       } else
       { Undo(*m);
 	PL_type_error("dict-key", pushWordAsTermRef(np));
 	popTermRef();
 
-	return FALSE;
+	return false;
       }
     } else if ( arityFunctor(f->definition) == 1 &&
 		(flags&DICT_GET_TERM) ) /* Name(Value) */
@@ -644,7 +715,7 @@ get_name_value(DECL_LD Word p, Word name, Word value, mark *m, int flags)
       *name = nameFunctor(f->definition);
       deRef2(&f->arguments[0], vp);
       *value = linkValI(vp);
-      return TRUE;
+      return true;
     }
   }
 
@@ -657,7 +728,7 @@ get_name_value(DECL_LD Word p, Word name, Word value, mark *m, int flags)
   PL_type_error(type, pushWordAsTermRef(p));
   popTermRef();
 
-  return FALSE;				/* type error */
+  return false;				/* type error */
 }
 
 
@@ -678,14 +749,14 @@ PL_is_dict(DECL_LD term_t t)
 
     if ( fd->name == ATOM_dict &&
 	 fd->arity%2 == 1 &&
-	 dict_ordered(f->arguments+1, fd->arity/2, &dupl) == TRUE )
-      return TRUE;
+	 dict_ordered(f->arguments+1, fd->arity/2, &dupl) == true )
+      return true;
   }
 
-  return FALSE;
+  return false;
 }
 
-API_STUB(int)
+API_STUB(bool)
 (PL_is_dict)(term_t t)
 ( valid_term_t(t);
   return PL_is_dict(t);
@@ -702,23 +773,23 @@ PL_get_dict_ex(term_t data, term_t tag, term_t dict, int flags)
 
   if ( PL_is_dict(data) )
   { PL_put_term(dict, data);
-    return TRUE;
+    return true;
   }
 
   if ( PL_is_list(data) )
-  { intptr_t len = lengthList(data, TRUE);
+  { intptr_t len = lengthList(data, true);
     Word ap, dp, tail;
     mark m;
     int rc;
 
     if ( len < 0 )
-      return FALSE;			/* not a proper list */
+      return false;			/* not a proper list */
 
     if ( unlikely(tTop+1 >= tMax) )
     { if ( !makeMoreStackSpace(TRAIL_OVERFLOW, ALLOW_GC|ALLOW_SHIFT) )
-	return FALSE;
+	return false;
     }
-    if ( (rc=ensureGlobalSpace(len*2+2, ALLOW_GC)) != TRUE )
+    if ( (rc=ensureGlobalSpace(len*2+2, ALLOW_GC)) != true )
       return raiseStackOverflow(rc);
     ap = gTop;
     Mark(m);
@@ -750,17 +821,17 @@ PL_get_dict_ex(term_t data, term_t tag, term_t dict, int flags)
     { Word head = HeadList(tail);
 
       if ( !get_name_value(head, ap+1, ap, &m, flags) )
-	return FALSE;
+	return false;
       ap += 2;
       tail = TailList(tail);
       deRef(tail);
     }
 
-    if ( (rc=dict_order(dp, &dupl)) == TRUE )
+    if ( (rc=dict_order(dp, &dupl)) == true )
     { gTop = ap;
       *valTermRef(dict) = consPtr(dp, TAG_COMPOUND|STG_GLOBAL);
       DEBUG(CHK_SECURE, checkStacks(NULL));
-      return TRUE;
+      return true;
     } else
     { term_t ex;
 
@@ -1046,14 +1117,14 @@ resortDictsInCodes(Code PC, Code end)
 	      default:
 	      { if ( kv_pos != kv_buf )
 		  free(kv_pos);
-		return TRUE;		/* not a dict */
+		return true;		/* not a dict */
 	      }
 	    }
 
 	    if ( !resortDictsInCodes(PCv, PC) )
 	    { if ( kv_pos != kv_buf )
 		free(kv_pos);
-	      return FALSE;
+	      return false;
 	    }
 
 	    PC = stepPC(PC);		/* skip key */
@@ -1097,7 +1168,7 @@ resortDictsInCodes(Code PC, Code end)
     }
   }
 
-  return TRUE;
+  return true;
 }
 
 int
@@ -1128,8 +1199,8 @@ right_arg:
     Word ea;
     word dupl;
 
-    if ( fd->name == ATOM_dict && fd->arity%2 == 1 &&
-	 dict_ordered(&t->arguments[1], fd->arity/2, &dupl) == FALSE )
+    if ( fd->name == ATOM_dict && fd->arity > 1 && fd->arity%2 == 1 &&
+	 dict_ordered(&t->arguments[1], fd->arity/2, &dupl) == false )
     { DEBUG(MSG_DICT, Sdprintf("Re-ordering dict\n"));
       dict_order((Word)t, &dupl);
     }
@@ -1171,18 +1242,7 @@ PRED_IMPL("is_dict", 1, is_dict, 0)
   Word p = valTermRef(A1);
 
   deRef(p);
-  if ( isTerm(*p) )
-  { Functor f = valueTerm(*p);
-    FunctorDef fd = valueFunctor(f->definition);
-    //word dupl;
-
-    if ( fd->name == ATOM_dict &&
-	 fd->arity%2 == 1 /*&&
-	 dict_ordered(f->arguments+1, fd->arity/2, &dupl) == TRUE*/ )
-      return TRUE;
-  }
-
-  return FALSE;
+  return is_dict_term(*p);
 }
 
 
@@ -1194,17 +1254,13 @@ PRED_IMPL("is_dict", 2, is_dict, 0)
   deRef(p);
   if ( isTerm(*p) )
   { Functor f = valueTerm(*p);
-    FunctorDef fd = valueFunctor(f->definition);
-    //word dupl;
 
-    if ( fd->name == ATOM_dict &&
-	 fd->arity%2 == 1 /*&&
-	 dict_ordered(f->arguments+1, fd->arity/2,  &dupl) == TRUE*/ )
+    if ( is_dict_functor(f->definition) )
       return unify_ptrs(&f->arguments[0], valTermRef(A2),
 			ALLOW_GC|ALLOW_SHIFT);
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1226,7 +1282,7 @@ pl_get_dict(term_t PL__t0, size_t PL__ac, int ex, control_t PL__ctx)
     { Word np = valTermRef(A1);
 
       if ( !get_dict_ex(A2, &dict, !ex) )
-	return FALSE;
+	return false;
 
       deRef(np);
       if ( is_dict_key(*np) )
@@ -1238,7 +1294,7 @@ pl_get_dict(term_t PL__t0, size_t PL__ac, int ex, control_t PL__ctx)
 	if ( ex )
 	  return PL_error(NULL, 0, NULL, ERR_EXISTENCE3,
 			  ATOM_key, A1, A2);
-	return FALSE;
+	return false;
       }
       if ( canBind(*np) )
       { i = 1;
@@ -1246,7 +1302,7 @@ pl_get_dict(term_t PL__t0, size_t PL__ac, int ex, control_t PL__ctx)
       }
       if ( !ex )
 	return PL_type_error("dict-key", A1);
-      return FALSE;
+      return false;
     }
     case FRG_REDO:
     { Functor f;
@@ -1276,32 +1332,32 @@ pl_get_dict(term_t PL__t0, size_t PL__ac, int ex, control_t PL__ctx)
 	    if ( i+2 < arity )
 	      ForeignRedoInt(i);
 	    else
-	      return TRUE;
+	      return true;
 	  } else if ( exception_term )
 	  { PL_close_foreign_frame(fid);
-	    return FALSE;
+	    return false;
 	  }
 	  PL_rewind_foreign_frame(fid);
 	}
 	PL_close_foreign_frame(fid);
       }
-      return FALSE;
+      return false;
     }
     default:
-      return TRUE;
+      return true;
   }
 }
 
 
 static
 PRED_IMPL("get_dict", 3, get_dict, PL_FA_NONDETERMINISTIC)
-{ return pl_get_dict(PL__t0, PL__ac, FALSE, PL__ctx);
+{ return pl_get_dict(PL__t0, PL__ac, false, PL__ctx);
 }
 
 
 static
 PRED_IMPL("get_dict_ex", 3, get_dict_ex, PL_FA_NONDETERMINISTIC)
-{ return pl_get_dict(PL__t0, PL__ac, TRUE, PL__ctx);
+{ return pl_get_dict(PL__t0, PL__ac, true, PL__ctx);
 }
 
 
@@ -1322,21 +1378,21 @@ PRED_IMPL("get_dict", 5, get_dict, 0)
        !(vp=dict_lookup_ptr(*valTermRef(dt), key)) ||
        !unify_ptrs(vp, valTermRef(A3), ALLOW_GC|ALLOW_SHIFT) ||
        !PL_put_term(av+0, A5) )
-    return FALSE;
+    return false;
 
   for(;;)
   { word new;
     int rc;
 
     if ( (rc = put_dict(*valTermRef(dt),
-			1, valTermRef(av), &new)) == TRUE )
+			1, valTermRef(av), &new)) == true )
     { term_t t = dt+3;
 
       *valTermRef(t) = new;
       return PL_unify(A4, t);
     } else
     { if ( !makeMoreStackSpace(rc, ALLOW_GC|ALLOW_SHIFT) )
-	return FALSE;
+	return false;
     }
   }
 }
@@ -1363,7 +1419,7 @@ PRED_IMPL("dict_create", 3, dict_create, 0)
   if ( PL_get_dict_ex(A3, A2, m, DICT_GET_ALL) )
     return PL_unify(A1, m);
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1408,12 +1464,12 @@ PRED_IMPL("dict_pairs", 3, dict_pairs, 0)
       ctx.head = PL_new_term_refs(2);
       ctx.tmp  = ctx.head+1;
 
-      if ( PL_get_arg(1, dict, ctx.tmp) &&
+      if ( _PL_get_arg(1, dict, ctx.tmp) &&
 	   PL_unify(ctx.tmp, A2) &&
 	   _PL_for_dict(dict, put_pair, &ctx, PL_FOR_DICT_SORTED) == 0 )
 	return PL_unify_nil_ex(ctx.tail);
 
-      return FALSE;
+      return false;
     }
   } else
   { term_t m = PL_new_term_ref();
@@ -1422,9 +1478,63 @@ PRED_IMPL("dict_pairs", 3, dict_pairs, 0)
       return PL_unify(A1, m);
   }
 
-  return FALSE;
+  return false;
 }
 
+/** dict_same_keys(?Dict1, ?Dict2) is semidet.
+*/
+
+#define unify_dict_copy(t, dt, dict)		\
+	LDFUNC(unify_dict_copy, t, dt, dict)
+
+static bool
+unify_dict_copy(DECL_LD term_t t, term_t dt, word dict)
+{ term_t tmp = PL_new_term_ref(); /* safe, we can allocate 10 */
+  word copy;
+  int rc;
+
+  for(;;)
+  { if ( (rc=copy_keys_dict(dict, &copy)) == true )
+    { *valTermRef(tmp) = copy;
+      return PL_unify(tmp, t);
+    }
+
+    if ( !makeMoreStackSpace(rc, ALLOW_SHIFT|ALLOW_GC) )
+      return false;
+
+    Word p = valTermRef(dt);
+    deRef(p);
+    dict = *p;
+  }
+}
+
+static
+PRED_IMPL("dict_same_keys", 2, dict_same_keys, 0)
+{ PRED_LD
+  Word d1 = valTermRef(A1);
+  Word d2 = valTermRef(A2);
+
+  deRef(d1);
+  deRef(d2);
+
+  if ( is_dict_term(*d1) )
+  { if ( is_dict_term(*d2) )
+      return same_keys_dict(*d1, *d2);
+    else if ( canBind(*d2) )
+      return unify_dict_copy(A2, A1, *d1);
+    else
+      return PL_type_error("dict", A2);
+  } else if ( is_dict_term(*d2) )
+  { if ( canBind(*d1) )
+      return unify_dict_copy(A1, A2, *d2);
+    else
+      return PL_type_error("dict", A1);
+  }
+
+  return PL_type_error("dict", A1);
+}
+
+
 
 /** put_dict(+New, +DictIn, -DictOut)
 
@@ -1449,7 +1559,7 @@ retry:
 
     if ( (rc = put_dict(*valTermRef(dt+0),
 			arity/2, &f2->arguments[1],
-			&new)) == TRUE )
+			&new)) == true )
     { term_t t = PL_new_term_ref();
 
       *valTermRef(t) = new;
@@ -1462,7 +1572,7 @@ retry:
     }
   }
 
-  return FALSE;
+  return false;
 }
 
 /** put_dict(+Key, +Dict0, +Value, -Dict)
@@ -1485,7 +1595,7 @@ retry:
     int rc;
 
     if ( (rc = put_dict(*valTermRef(dt),
-			1, valTermRef(av), &new)) == TRUE )
+			1, valTermRef(av), &new)) == true )
     { term_t t = PL_new_term_ref();
 
       *valTermRef(t) = new;
@@ -1498,7 +1608,7 @@ retry:
     }
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1533,7 +1643,7 @@ retry:
   { if ( !hasGlobalSpace(0) )
     { int rc;
 
-      if ( (rc=ensureGlobalSpace(0, ALLOW_GC)) != TRUE )
+      if ( (rc=ensureGlobalSpace(0, ALLOW_GC)) != true )
 	return raiseStackOverflow(rc);
       goto retry;
     }
@@ -1543,7 +1653,7 @@ retry:
       { term_t copy = PL_new_term_ref();
 
 	if ( !duplicate_term(value, copy) )
-	  return FALSE;
+	  return false;
 	value = copy;
 	val = valTermRef(value);
 	deRef(val);
@@ -1552,7 +1662,7 @@ retry:
     }
   }
 
-  if ( get_dict_ex(dict, &m, TRUE) &&
+  if ( get_dict_ex(dict, &m, true) &&
        get_name_ex(key, &k) )
   { Word vp;
 
@@ -1560,14 +1670,14 @@ retry:
     { if ( (flags&SETDICT_BACKTRACKABLE) )
 	TrailAssignment(vp);
       unify_vp(vp, val);
-      return TRUE;
+      return true;
     }
 
     return PL_error(NULL, 0, NULL, ERR_EXISTENCE3,
 		    ATOM_key, key, dict);
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1616,7 +1726,7 @@ retry:
     { int rc;
       word new;
 
-      if ( (rc=del_dict(*valTermRef(mt), key, &new)) == TRUE )
+      if ( (rc=del_dict(*valTermRef(mt), key, &new)) == true )
       { term_t t = PL_new_term_ref();
 
 	*valTermRef(t) = new;
@@ -1631,7 +1741,7 @@ retry:
     }
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1651,24 +1761,25 @@ retry:
   { int rc = select_dict(*valTermRef(dt+0), *valTermRef(dt+1), &r);
 
     switch(rc)
-    { case TRUE:
+    { case true:
       { term_t t = PL_new_term_ref();
 
 	*valTermRef(t) = r;
+	PL_check_data(t);
 	return PL_unify(A3, t);
       }
-      case FALSE:
+      case false:
 	return rc;
       case MEMORY_OVERFLOW:
 	return PL_no_memory();
       default:
 	if ( !makeMoreStackSpace(rc, ALLOW_GC|ALLOW_SHIFT) )
-	  return FALSE;
+	  return false;
         goto retry;
     }
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1683,19 +1794,19 @@ retry:
   { int rc = select_dict(*valTermRef(dt+0), *valTermRef(dt+1), NULL);
 
     switch(rc)
-    { case TRUE:
-      case FALSE:
+    { case true:
+      case false:
 	return rc;
       case MEMORY_OVERFLOW:
 	return PL_no_memory();
       default:
 	if ( !makeMoreStackSpace(rc, ALLOW_GC|ALLOW_SHIFT) )
-	  return FALSE;
+	  return false;
         goto retry;
     }
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1710,19 +1821,100 @@ retry:
   { int rc = partial_unify_dict(*valTermRef(dt+0), *valTermRef(dt+1));
 
     switch(rc)
-    { case TRUE:
-      case FALSE:
+    { case true:
+      case false:
 	return rc;
       case MEMORY_OVERFLOW:
 	return PL_no_memory();
       default:
 	if ( !makeMoreStackSpace(rc, ALLOW_GC|ALLOW_SHIFT) )
-	  return FALSE;
+	  return false;
         goto retry;
     }
   }
 
-  return FALSE;
+  return false;
+}
+
+/** '$get_dict_kv'(+Index, +Dict, -K, -V)
+    '$get_dict_kv'(+Index, +Dict1, +Dict2, -K, -V1, -V2)
+*/
+
+#define get_dict_kv(t0, arity)			\
+	LDFUNC(get_dict_kv, t0, arity)
+
+static bool
+get_dict_kv(DECL_LD term_t t0, int arity)
+{ size_t i;
+
+  if ( !PL_get_size_ex(t0, &i) || i == 0 )
+    return false;
+
+  Word p = valTermRef(t0+1);
+  deRef(p);
+  if ( isTerm(*p) )
+  { Functor f = valueTerm(*p);
+    size_t darity = arityFunctor(f->definition);
+
+    if ( !is_dict_functor(f->definition) )
+      return PL_type_error("dict", t0+1);
+
+    if ( i > darity/2 )
+      return false;
+    i--;
+
+    size_t dict_count = (arity-2)/2;
+    size_t key_arg = t0+1+dict_count;
+    term_t value = PL_new_term_ref();
+    term_t key = PL_new_term_ref();
+
+    for(size_t di=0; di < dict_count; di++)
+    { term_t dict = t0+1+di;
+
+      if ( di > 0 && !PL_is_functor(dict, f->definition) )
+      { if ( !PL_is_dict(dict) )
+	  return PL_type_error("dict", dict);
+	else
+	  return PL_domain_error("compatible_dict", dict);
+      }
+      _PL_get_arg(2+i*2, dict, value);
+      _PL_get_arg(3+i*2, dict, key);
+
+      if ( !PL_unify(key_arg, key) )
+      { if ( di == 0 )
+	  return false;
+	return PL_domain_error("compatible_dict", dict);
+      }
+
+      if ( !PL_unify(key_arg+1+di, value) )
+	return false;
+    }
+
+    return true;
+  }
+
+  return PL_type_error("dict", t0+1);
+}
+
+static
+PRED_IMPL("$get_dict_kv", 4, get_dict_kv, 0)
+{ PRED_LD
+
+  return get_dict_kv(A1, 4);
+}
+
+static
+PRED_IMPL("$get_dict_kv", 6, get_dict_kv, 0)
+{ PRED_LD
+
+  return get_dict_kv(A1, 6);
+}
+
+static
+PRED_IMPL("$get_dict_kv", 8, get_dict_kv, 0)
+{ PRED_LD
+
+  return get_dict_kv(A1, 8);
 }
 
 
@@ -1730,7 +1922,7 @@ retry:
 Part of FLI
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int
+bool
 PL_get_dict_key(atom_t key, term_t dict, term_t value)
 { GET_LD
   word d;
@@ -1740,14 +1932,14 @@ PL_get_dict_key(atom_t key, term_t dict, term_t value)
   valid_term_t(dict);
   valid_term_t(value);
 
-  if ( !get_dict_ex(dict, &d, FALSE) )
-    return FALSE;
+  if ( !get_dict_ex(dict, &d, false) )
+    return false;
   if ( (vp=dict_lookup_ptr(d, key)) )
   { *valTermRef(value) = linkValI(vp);
-    return TRUE;
+    return true;
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -1756,20 +1948,24 @@ PL_get_dict_key(atom_t key, term_t dict, term_t value)
 		 *******************************/
 
 BeginPredDefs(dict)
-  PRED_DEF("is_dict",	   1, is_dict,	    0)
-  PRED_DEF("is_dict",	   2, is_dict,	    0)
-  PRED_DEF("dict_create",  3, dict_create,  0)
-  PRED_DEF("dict_pairs",   3, dict_pairs,   0)
-  PRED_DEF("put_dict",	   3, put_dict,	    0)
-  PRED_DEF("put_dict",	   4, put_dict,	    0)
-  PRED_DEF("b_set_dict",   3, b_set_dict,   0)
-  PRED_DEF("nb_set_dict",  3, nb_set_dict,  0)
-  PRED_DEF("nb_link_dict", 3, nb_link_dict, 0)
-  PRED_DEF("get_dict",	   3, get_dict,	    PL_FA_NONDETERMINISTIC)
-  PRED_DEF("$get_dict_ex", 3, get_dict_ex,  PL_FA_NONDETERMINISTIC)
-  PRED_DEF("del_dict",	   4, del_dict,	    0)
-  PRED_DEF("get_dict",     5, get_dict,     0)
-  PRED_DEF("select_dict",  3, select_dict,  0)
-  PRED_DEF(":<",	   2, select_dict,  0)
-  PRED_DEF(">:<",	   2, punify_dict,  0)
+  PRED_DEF("is_dict",	     1, is_dict,	0)
+  PRED_DEF("is_dict",	     2, is_dict,	0)
+  PRED_DEF("dict_create",    3, dict_create,    0)
+  PRED_DEF("dict_pairs",     3, dict_pairs,     0)
+  PRED_DEF("dict_same_keys", 2, dict_same_keys, 0)
+  PRED_DEF("put_dict",	     3, put_dict,       0)
+  PRED_DEF("put_dict",	     4, put_dict,       0)
+  PRED_DEF("b_set_dict",     3, b_set_dict,     0)
+  PRED_DEF("nb_set_dict",    3, nb_set_dict,    0)
+  PRED_DEF("nb_link_dict",   3, nb_link_dict,   0)
+  PRED_DEF("get_dict",	     3, get_dict,	PL_FA_NONDETERMINISTIC)
+  PRED_DEF("$get_dict_ex",   3, get_dict_ex,    PL_FA_NONDETERMINISTIC)
+  PRED_DEF("del_dict",	     4, del_dict,	0)
+  PRED_DEF("get_dict",       5, get_dict,       0)
+  PRED_DEF("select_dict",    3, select_dict,    0)
+  PRED_DEF(":<",	     2, select_dict,    0)
+  PRED_DEF(">:<",	     2, punify_dict,    0)
+  PRED_DEF("$get_dict_kv",   4, get_dict_kv,    0)
+  PRED_DEF("$get_dict_kv",   6, get_dict_kv,    0)
+  PRED_DEF("$get_dict_kv",   8, get_dict_kv,    0)
 EndPredDefs
